@@ -2,13 +2,13 @@ package Egg::Plugin::SessionKit::Auth;
 #
 # Masatoshi Mizuno E<lt>lusheE<64>cpan.orgE<gt>
 #
-# $Id: Auth.pm 152 2007-05-16 22:55:52Z lushe $
+# $Id: Auth.pm 215 2007-11-06 23:17:11Z lushe $
 #
 use strict;
 use warnings;
 use base qw/Egg::Plugin::SessionKit/;
 
-our $VERSION = '2.01';
+our $VERSION = '2.10';
 
 =head1 NAME
 
@@ -25,18 +25,20 @@ Egg::Plugin::SessionKit::Auth - Authentication of session base.
     plugin_session=> {
       .......
       ...
-      psw_crypt_type => 'MD5',
-      uid_param_name => '__uid',
-      psw_param_name => '__psw',
-      data_path      => '<$e.dir.etc>/members.txt',
-      constant       => [qw/ uid psw active email nickname /],
-      uid_db_field   => 'uid',
-      psw_db_field   => 'psw',
-      active_db_field=> 'active',
-      messages=> {
-        uid_undefined => 'Please input ID.',
-        psw_undefined => 'Please input the password.',
-        ...
+      auth=> {
+        psw_crypt_type => 'MD5',
+        uid_param_name => '__uid',
+        psw_param_name => '__psw',
+        data_path      => '<$e.dir.etc>/members.txt',
+        constant       => [qw/ uid psw active email nickname /],
+        uid_db_field   => 'uid',
+        psw_db_field   => 'psw',
+        active_db_field=> 'active',
+        messages=> {
+          uid_undefined => 'Please input ID.',
+          psw_undefined => 'Please input the password.',
+          ...
+          },
         },
       },
   
@@ -49,13 +51,13 @@ Egg::Plugin::SessionKit::Auth - Authentication of session base.
 
 Example of authentication form. (L<Egg::View::Mason>)
 
-  % if (my $errmsg= $e->auth->errstr) {
-    <div class="error"><% $errmsg %></div>
+  % if (my $error_message = $e->auth->errstr) {
+    <div class="error"><% $error_message %></div>
   % }
   <form method="POST" action="/auth">
   <input type="hidden" name="ticket" value="<% $e->ticket_id(1) %>" />
-  USER-ID : <input type="text" name="__uid" /> <br />
-  PASSWORD: <input type="text" name="__psw" /> <br />
+  USER-ID : <input name="__uid" type="text" /> <br />
+  PASSWORD: <input name="__psw" type="password" /> <br />
   <input type="submit" />
   </form>
 
@@ -63,33 +65,34 @@ Example of code.
 
   # The Auth object is acquired.
   my $auth= $e->auth;
-  
+
+  # The input of the login form is checked.
+  my $user_id;
+  if ($e->ticket_check and  $user_id= $e->auth->login) {
+     $e->response->redirect('/members');
+  } else {
+     Login form is displayed again.
+  }
+
   # The user who is logging it in now is checked.
-  if (my $uid= $e->user_name) {
-    print "is login: $uid";
+  if (my $user_id= $e->user_name) {
+    print "is login: $user_id";
   } else {
     print "It doesn't login.";
   }
-  
-  # The input of the login form is checked.
-  if (my $user= $e->auth->login) {
-    ..... code after it login.
-  } else {
-    $e->response->redirect('/auth');
-  }
-  
+
   # Refer to user's data after it logs it in.
   my $user= $e->auth->user;
   print " NickName : $user->{nickname} \n";
   print " E-mail   : $user->{email} \n";
-  
+
   # The data of an arbitrary user is acquired.
   if (my $user= $e->auth->restore($user_id)) {
     print " NickName : $user->{nickname} ";
   } else {
     print "There is no registration.";
   }
-  
+
   # Logout.
   $e->auth->logout;
 
@@ -101,7 +104,7 @@ It is a plugin that offers the attestation function of the session base.
 
 =head1 CONFIGURATION
 
-Please define it in 'plugin_session' with HASH.
+Please define it in 'plugin_session->{auth}' with HASH.
 
 =head2 uid_param_name
 
@@ -172,44 +175,42 @@ The following subclasses are included in the standard.
   L<Egg::Plugin::SessionKit::Auth::DBI>,
   L<Egg::Plugin::SessionKit::Auth::DBIC>,
 
-=head1 METHODS
-
-=head2 auth
-
-The handler object is returned.
-
 =cut
-sub _setup {
-	my($e)= @_;
-	my $handler= $e->{session_auth_handler}
-	          || die q{ Mistake of method of loading plugin. };
-	my $sscf= $e->config->{plugin_session} ||= {};
-	my $conf= $sscf->{auth} ||= {};
-	$handler->_startup($e, $conf);
-	no warnings 'redefine';
-	*auth= sub { $_[0]->{auth} ||= $handler->new($_[0], $conf) };
-	$e->next::method;
-}
 
-=head2 user_name
+{
+	my($handler, $aconf);
+	sub _setup {
+		my($e)= @_;
+		$handler= $e->{session_auth_handler}
+		   || die q{ Mistake of method of loading plugin. };
+		my $cf= $e->config->{plugin_session} ||= {};
+		$aconf= $cf->{auth} ||= {};
+		##
+		$aconf->{uid_param_name}  ||= '__uid';
+		$aconf->{psw_param_name}  ||= '__psw';
+		$aconf->{uid_db_field}    || die qq{ Please setup uid_db_field. };
+		$aconf->{psw_db_field}    || die qq{ Please setup psw_db_field. };
+		$aconf->{active_db_field} ||= 0;
+		$aconf->{psw_crypt_type}  ||= 'Plain';
+		##
+		$handler->_startup($e, $aconf);
+		$e->next::method;
+	}
+	sub auth { $_[0]->{sessionkit_auth} ||= $handler->new($_[0], $aconf) }
+	sub reset_auth { $_[0]->{sessionkit_auth}= $handler->new($_[0], $aconf) }
+  };
 
-It is an accessor to $e-E<gt>auth-E<gt>user_name.
-
-=cut
 sub user_name { $_[0]->auth->user_name }
+sub user_data { $_[0]->auth->user_data }
 
 package Egg::Plugin::SessionKit::Auth::handler;
 use strict;
 use warnings;
-use Class::C3;
 use UNIVERSAL::require;
 use base qw/Class::Accessor::Fast/;
 
-__PACKAGE__->mk_accessors( qw/e config messages session/ );
+__PACKAGE__->mk_accessors( qw/e config messages/ );
 
-=head1 HANDLER METHODS
-
-=cut
 {
 	no strict 'refs';  ## no critic
 	no warnings 'redefine';
@@ -229,27 +230,10 @@ sub _startup {
 	$pkg->require or die $@;
 	@_;
 }
-sub _initialize {
-	my($class, $conf)= @_;
-	$conf->{uid_param_name}  ||= '__uid';
-	$conf->{psw_param_name}  ||= '__psw';
-	$conf->{uid_db_field}    || die qq{ Please setup uid_db_field. };
-	$conf->{psw_db_field}    || die qq{ Please setup psw_db_field. };
-	$conf->{active_db_field} ||= 0;
-	$conf->{psw_crypt_type}  ||= 'Plain';
-	@_;
-}
-
-=head2 new
-
-Constructor.
-
-=cut
 sub new {
 	my($class, $e, $conf)= @_;
 	bless {  e => $e,
 	  config   => $conf,
-	  session  => $e->session,
 	  messages => ($conf->{messages} || {}),
 	  errstr   => '',
 	  uid_param_name  => $conf->{uid_param_name},
@@ -259,6 +243,106 @@ sub new {
 	  active_db_field => $conf->{active_db_field},
 	  }, $class;
 }
+sub session {
+	$_[0]->e->session;
+}
+sub login {
+	my $auth= shift;
+	my $data= $auth->_get_login_argument(@_) || return 0;
+	my $user= $auth->restore($data->[0])
+	       || return $auth->error_no_regist;
+	my $crypt= $user->{$auth->{psw_db_field}}
+	       || return $auth->error_unset_psw;
+	return $auth->error_unactive
+	  if ($auth->{active_db_field} && ! $user->{$auth->{active_db_field}});
+	return $auth->error_discord_psw unless $auth->psw_check(@$data, $crypt);
+	my $uid= $user->{$auth->{uid_param_name}}= $user->{$auth->{uid_db_field}};
+	$auth->user_data($user);
+	$auth->e->debug_out("# + session auth : [$uid] - Login is succeed.");
+	$user;
+}
+sub logout {
+	my($auth)= @_;
+	my $uid= $auth->user_name || return 1;
+	$auth->session->clear;
+	undef $auth->{user_name};
+	$auth->e->debug_out("# + session auth : [$uid] - Logout OK.");
+	return 1;
+}
+sub reset {
+	my $auth= shift;
+	my $user= shift || die q{ I want user data. };
+	undef $auth->{user_name} if $auth->{user_name};
+	if (my $uid= $user->{$auth->{uid_db_field}})
+	   { $user->{$auth->{uid_param_name}}= $uid }
+	$user;
+}
+sub user_name {
+	$_[0]->{user_name} ||= $_[0]->user->{$_[0]->{uid_param_name}} || 0;
+}
+
+sub user_data_key_name { 'session_user_data' }
+
+sub user {
+	$_[0]->session->{ $_[0]->user_data_key_name } || {};
+}
+sub user_data {
+	my $auth= shift;
+	return $auth->session->{ $auth->user_data_key_name } || {} unless @_;
+	$auth->session->{ $auth->user_data_key_name }= shift;
+}
+
+sub get_uid_param {
+	$_[0]->e->request->params->{$_[0]->{uid_param_name}} || "";
+}
+sub get_psw_param {
+	$_[0]->e->request->params->{$_[0]->{psw_param_name}} || "";
+}
+sub error {
+	my $auth= shift;
+	$auth->{errstr}= shift || 'internal_error';
+	return 0;
+}
+*errstr= \&error_message;
+sub error_message {
+	my($auth)= @_;
+	return $auth->{errstr}
+	  ? ($auth->messages->{$auth->{errstr}} || $auth->{errstr}): 0;
+}
+sub _get_login_argument {
+	my $auth= shift;
+	$auth->_https_check || return $auth->error_secure_onry;
+	my $uid= shift || $auth->get_uid_param
+	      || return $auth->error_uid_undefined;
+	my $psw= shift || $auth->get_psw_param
+	      || return $auth->error_psw_undefined;
+	s/\s+//g for ($uid, $psw);
+	[$uid, $psw];
+}
+
+=head1 METHODS
+
+=head2 auth
+
+The handler object is returned.
+
+=head2 user_name
+
+It is an accessor to $e-E<gt>auth-E<gt>user_name.
+
+=head2 user_data
+
+Other data of the user under login is returned by the HASH reference.
+
+=head2 reset_auth
+
+It tries to make the object of the module of this now newly.
+
+=head1 HANDLER METHODS
+
+=head2 new
+
+Constructor.
 
 =head2 login ( [USER_ID], [LOGIN_PASSWD] )
 
@@ -276,97 +360,38 @@ When LOGIN_PASSWD is omitted, it acquires it from 'get_psw_param' method.
     .....
     ...
 
-=cut
-sub login {
-	my $auth= shift;
-	my $data= $auth->_get_login_argument(@_) || return 0;
-	my $user= $auth->restore($data->[0])
-	       || return $auth->error_no_regist;
-	my $crypt= $user->{$auth->{psw_db_field}}
-	       || return $auth->error_unset_psw;
-	return $auth->error_unactive
-	  if ($auth->{active_db_field} && ! $user->{$auth->{active_db_field}});
-	return $auth->error_discord_psw unless $auth->psw_check(@$data, $crypt);
-	my $uid= $user->{$auth->{uid_param_name}}= $user->{$auth->{uid_db_field}};
-	$auth->session->{auth_data}= $user;
-	$auth->e->debug_out("# + session auth : [$uid] - Login is succeed.");
-	$user;
-}
-
 =head2 logout
 
 It logs out if it is login.
 
-=cut
-sub logout {
-	my($auth)= @_;
-	my $uid= $auth->user_name || return 1;
-	tied(%{$auth->session})->clear;
-	undef($auth->{user_name});
-	$auth->e->debug_out("# + session auth : [$uid] - Logout OK.");
-	return 1;
-}
-
 =head2 user
 
 User's registration data is returned by the HASH reference if it is logging it in.
 
   my $nickname= $e->auth->user->{nickname};
 
-=cut
-sub user { $_[0]->session->{auth_data} || {} }
-
 =head2 user_name
 
 User ID that succeeds in the attestation is returned.
 
 * 0 returns when failing in the attestation.
 
-=cut
-sub user_name {
-	$_[0]->{user_name} ||= $_[0]->user->{$_[0]->{uid_param_name}} || 0;
-}
-
 =head2 get_uid_param
 
 User ID is returned from the form data based on 'uid_param_name'.
 
-=cut
-sub get_uid_param {
-	$_[0]->e->request->params->{$_[0]->{uid_param_name}} || "";
-}
-
 =head2 get_psw_param
 
 The login password is returned from the form data based on 'psw_param_name'.
 
-=cut
-sub get_psw_param {
-	$_[0]->e->request->params->{$_[0]->{psw_param_name}} || "";
-}
-
 =head2 error ( [ERROR_MESSAGE] )
 
 The error message is stored.
 
-=cut
-sub error {
-	my $auth= shift;
-	$auth->{errstr}= shift || 'internal_error';
-	return 0;
-}
-
 =head2 errstr
 
 The error set by 'error' method is returned by the message for the screen output.
 
-=cut
-sub errstr {
-	my($auth)= @_;
-	return $auth->{errstr}
-	  ? ($auth->messages->{$auth->{errstr}} || $auth->{errstr}): 0;
-}
-
 =head2 ... etc. ( error methods ),
 
   error_uid_undefined
@@ -380,19 +405,6 @@ The above-mentioned method is contained as an accessor of 'error' method.
 A prescribed error is set only by calling this method.
 The above-mentioned method always returns 0.
 
-=cut
-
-sub _get_login_argument {
-	my $auth= shift;
-	$auth->_https_check || return $auth->error_secure_onry;
-	my $uid= shift || $auth->get_uid_param
-	      || return $auth->error_uid_undefined;
-	my $psw= shift || $auth->get_psw_param
-	      || return $auth->error_psw_undefined;
-	s/\s+//g for ($uid, $psw);
-	[$uid, $psw];
-}
-
 =head1 WARNING
 
 After the attestation succeeds, the acquired data is preserved in the session.
